// sprite_handler class implementation file.

#include "sprite_handler.h"

Sprite_Handler::Sprite_Handler()
{

  //takes in command line to set up the screen or defaults if there is none.
  //returns a false on error and true on success
  
  std::string title_string = "RPG GFX Alpha";
  std::string icon_string = "RPG GFX Alpha - Minimized";
  char try_again;
  
  //clear console for debugging output
  system("reset");
  
  
  //initializing SDL for graphics abilities...
  std::cout << "Initializing SDL...";
  if (SDL_Init(SDL_INIT_VIDEO) < 0)
  {
    fprintf(stderr, "\nERROR: Could not initialize SDL: %s\n", SDL_GetError());
    exit(1);
  }
  
  else
  {
    std::cout << "Done!\n";
    atexit(SDL_Quit);
  }
       
  std::cout << "Setting up window for application on hardware surface...";
  
  
  //set up a simple screen that supports any bitdepth on the hardaware surface.
  //if that fails, it will prompt to try the software surface. If this fails the program
  //bails out.
  
  screen = SDL_SetVideoMode(640, 480, 16, SDL_SWSURFACE | SDL_ANYFORMAT);//SDL_HWSURFACE | SDL_ANYFORMAT | SDL_DOUBLEBUF | SDL_FULLSCREEN);
  if (!screen)
  {
    fprintf(stderr, "\nERROR: Could not set up screen on hardware surface: %s\n", SDL_GetError());
    std::cout << "\nTry again using a software surface? (Warning: Program may run slower!)[y/n]: ";
    std::cin >> try_again;
    
    if ((try_again == 'y') || (try_again == 'Y'))
    {
      screen = SDL_SetVideoMode(640, 480, 16, SDL_SWSURFACE | SDL_ANYFORMAT);
      if (!screen)
      {
        fprintf(stderr, "\nERROR: Could not set up screen on software surface: %s\n", SDL_GetError());
	std::cout << "Out of options, program must terminate. Please report this error to website!\n";
        exit(1);
      }
    }
    else
    {
      std::cout << "Not trying software surface, program terminating...\n";
      exit(1);
    }  
  }
  
  else
    std::cout << "Done!\n";
    
  //setting window's caption:
  std::cout << "Setting window caption and minimized caption...";
  SDL_WM_SetCaption(title_string.c_str(), icon_string.c_str());
  std::cout << "Done!\n";
  std::cout << "Testing captions...\n";
//  SDL_WM_GetCaption(title_string.c_str(), icon_string.c_str());
//  if (title_string)
//    std::cout << "Got title_string: " << title_string << std::endl;
//  if (icon_string)
//    std::cout << "Got icon_string: " << icon_string << std::endl;
  
  //clearing the screen
  std::cout << "Clearing the screen so it's fresh...";
  SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format,0,0,0));
  std::cout << "Done!\n";
  
  //setting up sprite information:
  //animation etc...
  if (!init_sprites())
  {
    std::cout << "ERROR: Could not initialize sprites needed for program!\nTerminating...\n";
    exit(1);
  }
  
  //setting transparency color key
  if (!set_colorKey())
  {
    std::cout << "\nERROR: Could not set color key for sprites!\nTerminating...\n";
    exit(1);
  }
  
  //removing the cursor...
  std::cout << "Removing the cursor...";
  SDL_ShowCursor(false);
  std::cout << "Done!\n";
  
  std::cout << "Initializing Font system...";
  if (!init_fonts())
  {
    std::cout << "\nERROR: Fatal error occurred while initializing fonts!\nProgram must terminate!\n";
    exit(1);
  }
  
  std::cout << "Initialization complete!\nStarting program...\n";

    


}

Sprite_Handler::~Sprite_Handler()
{
  //this is giving me segmentation faults. I'm thinking that SDL_Quit is taking
  //care of deleting these b/c they're SDL_Surface*'s 
 // delete [] player_obj.sprite_surface;
//  delete [] bg_obj.sprite_surface;

 SDL_FreeSurface(screen);
 SDL_FreeSurface(fps.message);
 SDL_FreeSurface(player_obj.sprite_surface[0]);
 SDL_FreeSurface(player_obj.sprite_surface[1]);
 SDL_FreeSurface(enemy_obj.sprite_surface[0]);
 SDL_FreeSurface(bg_obj.sprite_surface[0]);
  
  TTF_Quit();
  SDL_Quit();
}


bool Sprite_Handler::collision_detection(Sprite &in1, Sprite &in2)
{
  //sprites are passed in via reference so we know who we're talking about.

  SDL_Rect rect1;
  SDL_Rect rect2;
  
  //setting the rect information based on the surfaces sent to be checked.
  rect1.x = in1.x;
  rect1.y = in1.y;
  rect1.h = in1.sprite_surface[0]->h;
  rect1.w = in1.sprite_surface[0]->w;
  
  rect2.x = in2.x;
  rect2.y = in2.y;
  rect2.h = in2.sprite_surface[0]->h;
  rect2.w = in2.sprite_surface[0]->w;


  //check to see if boxes are touching on any side...
  //these statements would be false if they are.
  
  if(rect2.x + rect2.w < rect1.x)	 //rect2's right side not touching rect1's...
    return false;	

  if(rect2.x > rect1.x + rect1.w)	 //rect1's right side not touching rect2's...
    return false;

  if(rect2.y + rect2.h < rect1.y)        //rect 2's top side not touching rect1's...
    return false;
	
  if(rect2.y > rect1.y + rect1.h)        //rect1's top side not touching rect2's...
    return false;


  //if they haven't been thrown back yet, they must be touching.
  return true;
}

bool Sprite_Handler::draw_scene(short x_moved, short y_moved)
{
  //draws the images to the screen and locks it if necessary before doing so.
  
  //locking if necessary...
  if (SDL_MUSTLOCK(screen))
  {
    if (SDL_LockSurface(screen) < 0)
      return false;
  }
  

  //drawing the screen.
  if (!bg_obj.drawn)
  {
    draw_sprite(bg_obj.sprite_surface[0], bg_obj.x, bg_obj.y);
    bg_obj.drawn = true;
  }
  
  draw_part_sprite(enemy_obj.sprite_surface[enemy_obj.ani_frame], enemy_obj.x, enemy_obj.y, 1, -1);


  //draws player object at current x&y location and masks the former location of where
  //the player was. (Via x_moved&y_moved arguements).
  draw_part_sprite(player_obj.sprite_surface[player_obj.ani_frame], player_obj.x, player_obj.y, x_moved, y_moved); //draws bitmap test sprite
 
  if (!fps.drawn)
  {
    draw_part_sprite(fps.message, fps.x, fps.y, 0, 0);
    fps.drawn = true;
  }
  
  //unlock the screen in need be:
  if (SDL_MUSTLOCK(screen))
    SDL_UnlockSurface(screen);
    
  //flip the buffers so we see the image.
  SDL_Flip(screen);
  
  //check for collision detection.
  if (collision_detection(enemy_obj, player_obj))
    return false;
  
  else   
    return true;
}

void Sprite_Handler::draw_sprite(SDL_Surface *surface, short x, short y)
{
  SDL_Rect destination;
  
  //set draw locations...
  destination.x = x;
  destination.y = y;
  
  //draw given surface, in entirety, on the main screen, at the destination given.
  SDL_BlitSurface(surface, NULL, screen, &destination);
}

void Sprite_Handler::draw_part_sprite(SDL_Surface *surface, short x, short y, short x_moved, short y_moved)
{
  //masks background over former location where something was and then redraws
  //the surface at it's current location.
  //this is used so we don't have to redraw the whole background.
  //there's a HUGE speed increase by doing that. (about 10+ times)
     
  //declare rects to be used.
  SDL_Rect destination_img;
  SDL_Rect destination_surface;

  //mask the old at past image location.
  destination_img.x = x + x_moved;
  destination_img.y = y + y_moved;
  destination_img.w = surface->w;
  destination_img.h = surface->h;
  
  destination_surface.x = x + x_moved;
  destination_surface.y = y + y_moved;
  destination_surface.w = surface->w; //width; //width;
  destination_surface.h = surface->h; //height; //height;

  SDL_BlitSurface(bg_obj.sprite_surface[0], &destination_img, screen, &destination_surface);

  //draw new image at current location.
  destination_img.x = 0;
  destination_img.y = 0;
  destination_img.w = surface->w; //width; //width;
  destination_img.h = surface->h; //height; //height;
  
  destination_surface.x = x; //x;
  destination_surface.y = y; //y;
  
  SDL_BlitSurface(surface, &destination_img, screen, &destination_surface);
  
}

bool Sprite_Handler::init_sprites(void)
{
  //switched to a struct. I'm thinking this was a smart idea.
  
  //dynamically allocating frames for each sprite struct so that you can use the
  //same struct for however many frames of animation you want for it.
  //# frames = array indices + 1
  player_obj.sprite_surface = new SDL_Surface*[1]; //2 frames of animation
  bg_obj.sprite_surface = new SDL_Surface*[0];     //1 frame of animation (static)
  enemy_obj.sprite_surface = new SDL_Surface*[1];  //2 frames of animation
  
  //set images to each frame of the surface...
  player_obj.sprite_surface[0] = SDL_LoadBMP("./images/p_1.bmp");
  player_obj.sprite_surface[1] = SDL_LoadBMP("./images/p_2.bmp");

  bg_obj.sprite_surface[0] = SDL_LoadBMP("./images/background_test_small.bmp");
  
  enemy_obj.sprite_surface[0] = SDL_LoadBMP("./images/pepper_enemy.bmp");
  enemy_obj.sprite_surface[1] = SDL_LoadBMP("./images/pepper_enemy2.bmp");
  

  //make sure everything loaded alright..,
  if (!player_obj.sprite_surface[0])
    return false;
  if (!player_obj.sprite_surface[1])
    return false;
  if (!bg_obj.sprite_surface[0])
    return false;
  if (!enemy_obj.sprite_surface[0])
    return false;
  if (!enemy_obj.sprite_surface[1])
    return false;  
  
  
  //set sprite locations...
  player_obj.x = 250;
  player_obj.y = 150;
  
  bg_obj.x = 0;
  bg_obj.y = 0;
  
  enemy_obj.x = 290;
  enemy_obj.y = 300;

  //animation initialization
  //player
  player_obj.ani_frame = 0;
  player_obj.ani_status = 0;
  player_obj.ani_wait_count = 0;
  
  //enemy
  enemy_obj.ani_frame = 0;
  enemy_obj.ani_status = 0;
  enemy_obj.ani_wait_count = 0;
  
  //setting up drawn statistics.
  bg_obj.drawn = false;
  enemy_obj.drawn = false;
  player_obj.drawn = true; //just so it has a value.
  
  return true;
}

bool Sprite_Handler::set_colorKey(void)
{
  std::cout << "Setting transparence color to FF00FF...";
  colorKey = SDL_MapRGB(screen->format, 255, 0, 255);
  
  if (colorKey > 0)
    std::cout << "Done!\n";
  else
  {
    std::cout << "\n\nWARNING: SDL_MapRGB(SDL_PixelFormat*, Uint8, Uint8, Uint8) returned a zero "
                 "or negative value. Continuing program, but it may not run correctly...\n";
    return false;
  }

  std::cout << "Setting transparency color key to sprites...";
  SDL_SetColorKey(player_obj.sprite_surface[0], SDL_SRCCOLORKEY | SDL_RLEACCEL, colorKey);
  SDL_SetColorKey(player_obj.sprite_surface[1], SDL_SRCCOLORKEY | SDL_RLEACCEL, colorKey);
  SDL_SetColorKey(enemy_obj.sprite_surface[0], SDL_SRCCOLORKEY | SDL_RLEACCEL, colorKey);
  SDL_SetColorKey(enemy_obj.sprite_surface[1], SDL_SRCCOLORKEY | SDL_RLEACCEL, colorKey);  
  std::cout << "Done!\n";
  
  return true;
}

bool Sprite_Handler::init_fonts(void)
{ 

  fps.msg_edit = "FPS: ";
  
  //the only way I could think of initializing it...
  SDL_Color tempColor = {255, 255, 255};

  //set up SDL_ttf to be used..       
  if (TTF_Init() < 0)
    return false;

  //set text color of the text.
  fps.textColor = tempColor;
  
  //open the font.
  fps.font = TTF_OpenFont("./fonts/courier.ttf", 12);
  
  if (fps.font == NULL)
    return false;

  //set the message of the font.    
  fps.message = TTF_RenderText_Solid(fps.font, fps.msg_edit.c_str(), fps.textColor);
  
  if (fps.message == NULL)
    return false;
  
  //set location to blit the text.
  fps.x = 500;
  fps.y = 5;
  
  fps.drawn = false;
  
  return true;
}

void Sprite_Handler::updateFPS(std::string fps_in)
{
  //this updates the fps text surface with the current frames
  //per second calculated by the fps_handler class.
  
  fps.msg_edit = fps_in;
  
  //set the message of the font.    
  fps.message = TTF_RenderText_Solid(fps.font, fps.msg_edit.c_str(), fps.textColor);
  
  //some error checking.
  if (fps.message == NULL)
    std::cout << "ERROR: Could not render FPS text!\n";

   
  fps.drawn = false;
  
}
